@@ -32,6 +32,7 @@ from gwaslab.filtervalue import inferbuild
 from gwaslab.filtervalue import sampling
 from gwaslab.mqqplot import mqqplot
 from gwaslab.calculate_gc import lambdaGC
+from gwaslab.h2_conversion import _get_per_snp_r2
 from gwaslab.getsig import getsig
 from gwaslab.getdensity import getsignaldensity
 from gwaslab.getdensity import assigndensity
@@ -420,8 +421,7 @@ class Sumstats():
         # return sumstats object    
         return output
 
-    def get_density(self, sig_list=None, windowsizekb=100,
-                    large_number=10000000000,**args):
+    def get_density(self, sig_list=None, windowsizekb=100,**args):
         
         if "SNPID" in self.data.columns:
             id_to_use = "SNPID"
@@ -434,7 +434,6 @@ class Sumstats():
                                                     chrom="CHR",
                                                     pos="POS",
                                                     bwindowsizekb=windowsizekb,
-                                                    large_number=large_number,
                                                     log=self.log)
         else:
             if isinstance(sig_list, pd.DataFrame):
@@ -475,8 +474,8 @@ class Sumstats():
                            **args)
         return output
         
-    def get_per_snp_h2(self,**args):
-        self.data = getpersnph2(self.data,beta="BETA",af="EAF",**args)
+    def get_per_snp_r2(self,**args):
+        self.data = _get_per_snp_r2(self.data, beta="BETA", af="EAF", n="N", log=self.log, **args)
         #add data inplace
 
     
