@@ -5,13 +5,6 @@ import seaborn as sns
 import numpy as np
 import scipy as sp
 import gwaslab as gl
-from gwaslab.Log import Log
-from gwaslab.getsig import getsig
-from gwaslab.getsig import annogene
-from gwaslab.CommonData import get_chr_to_number
-from gwaslab.CommonData import get_number_to_chr
-from gwaslab.CommonData import get_recombination_rate
-from gwaslab.CommonData import get_gtf
 from pyensembl import EnsemblRelease
 from allel import GenotypeArray
 from allel import read_vcf
@@ -20,6 +13,29 @@ import matplotlib as mpl
 from scipy import stats
 from mpl_toolkits.axes_grid1.inset_locator import inset_axes
 import gc as garbage_collect
+from adjustText import adjust_text
+from gwaslab.Log import Log
+from gwaslab.getsig import getsig
+from gwaslab.getsig import annogene
+from gwaslab.CommonData import get_chr_to_number
+from gwaslab.CommonData import get_number_to_chr
+from gwaslab.CommonData import get_recombination_rate
+from gwaslab.CommonData import get_gtf
+from gwaslab.textreposition import adjust_text_position
+from gwaslab.quickfix import _quick_fix
+#from gwaslab.quickfix import _quick_fix_p
+#from gwaslab.quickfix import _quick_fix_mlog10p
+#from gwaslab.quickfix import _quick_fix_chr
+#from gwaslab.quickfix import _quick_fix_pos
+#from gwaslab.quickfix import _quick_fix_eaf
+from gwaslab.quickfix import _get_largenumber
+from gwaslab.quickfix import _quick_add_tchrpos
+from gwaslab.quickfix import _quick_merge_sumstats
+from gwaslab.quickfix import _quick_assign_i
+from gwaslab.quickfix import _quick_extract_snp_in_region
+from gwaslab.quickfix import _quick_assign_highlight_hue_pair
+from gwaslab.quickfix import _quick_assign_marker_relative_size
+from gwaslab.annotateplot import annotate_pair
 
 def plot_miami( 
           path1,
@@ -27,34 +43,43 @@ def plot_miami(
           cols1=["CHR","POS","P"],
           cols2=["CHR","POS","P"],
           sep=["\t","\t"],
+          chr_dict  = get_chr_to_number(),
+          chr_dict1 = False,
+          chr_dict2 = False,
+          scaled=False,
+          scaled1=False,
+          scaled2=False,
           region=None,
           region_step = 21,
           region_grid = False,
           region_grid_line = {"linewidth": 2,"linestyle":"--"},
           region_lead_grid = True,
           region_lead_grid_line = {"alpha":0.5,"linewidth" : 2,"linestyle":"--","color":"#FF0000"},
-          anno=None,
-          anno_set=[],
-          anno_set1=[],
-          anno_set2=[],
-          anno_alias1={},
-          anno_alias2={},
-          anno_d1={},
-          anno_d2={},
-          anno_args={},
+          anno = None,
+          anno_set=list(),
+          anno_set1=list(),
+          anno_set2=list(),
+          anno_alias1=dict(),
+          anno_alias2=dict(),
+          anno_d1=dict(),
+          anno_d2=dict(),
+          anno_args=dict(),
+          anno_style="right",
           anno_fixed_arm_length=None,
           anno_source = "ensembl",
+          anno_max_iter=100,
+          anno_adjust=False,
           arm_offset=50,
           arm_scale=1,
           arm_scale_d=None,
-          highlight  = [],
-          highlight1 = [],
-          highlight2 = [],
+          highlight  = list(),
+          highlight1 = list(),
+          highlight2 = list(),
           highlight_color="#CB132D",
           highlight_windowkb = 500,
-          pinpoint=[],
-          pinpoint1=[],
-          pinpoint2=[],
+          pinpoint=list(),
+          pinpoint1=list(),
+          pinpoint2=list(),
           pinpoint_color ="red",
           titles=["",""],
           titles_pad=[0.2,0.2], 
@@ -77,13 +102,12 @@ def plot_miami(
           scatter_kwargs={},
           marker_size=(5,25),
           verbose=True,
-          large_number=10000000000,
           repel_force=0.03,
           save=None,
           saveargs={"dpi":100,"facecolor":"white"},
           log=Log()
           ):
-    ## load sumstats1 ###########################################################################################################
+    ## figuring arguments ###########################################################################################################
     if verbose: log.write("Start to plot miami plot with the following basic settings:")
     if verbose: log.write(" -Genome-wide significance level is set to "+str(sig_level)+" ...")
     if len(anno_set)>0 :
@@ -97,7 +121,46 @@ def plot_miami(
         if verbose: log.write(" -Region to plot : chr"+str(region[0])+":"+str(region[1])+"-"+str(region[2])+".")  
     if dpi!=100:
         figargs["dpi"] = dpi
-        
+    
+    if chr_dict1==False:
+        chr_dict1 = chr_dict
+    if chr_dict2==False:
+        chr_dict2 = chr_dict
+    if scaled == True:
+        scaled1 = True
+        scaled2 = True
+    chrom = "CHR"
+    pos="POS"
+    
+    ## load sumstats1 ###########################################################################################################
+    if verbose: log.write(" -Loading sumstats1:" + path1)
+    if verbose: log.write(" -Sumstats1 CHR,POS,P information will be obtained from:",cols1)
+    sumstats1 = pd.read_csv(path1,sep=sep[0],usecols=cols1,dtype={cols1[0]:"string",cols1[1]:"Int64",cols1[2]:"float64"},**readcsv_args)
+    sumstats1 = sumstats1.rename(columns={cols1[0]:"CHR",cols1[1]:"POS",cols1[2]:"P"})
+    sumstats1 = _quick_fix(sumstats1,chr_dict=chr_dict1, scaled=scaled1, verbose=verbose, log=log)
+
+    ## load sumstats2 ###########################################################################################################
+    if verbose: log.write(" -Loading sumstats2:" + path2)
+    if verbose: log.write(" -Sumstats2 CHR,POS,P information will be obtained from:",cols2)
+    sumstats2 = pd.read_csv(path2,sep=sep[1],usecols=cols2,dtype={cols1[0]:"string",cols1[1]:"Int64",cols1[2]:"float64"},**readcsv_args)
+    sumstats2 = sumstats2.rename(columns={cols2[0]:"CHR",cols2[1]:"POS",cols2[2]:"P"})
+    sumstats2 = _quick_fix(sumstats2,chr_dict=chr_dict2, scaled=scaled2, verbose=verbose, log=log)
+
+    # get a large number
+    large_number = _get_largenumber(sumstats1["POS"].max(), sumstats2["POS"].max(),log=log)
+    
+    ## create merge index and then merge
+    sumstats1 = _quick_add_tchrpos(sumstats1,large_number=large_number, dropchrpos=False, verbose=verbose, log=log)
+    sumstats2 = _quick_add_tchrpos(sumstats2,large_number=large_number, dropchrpos=False, verbose=verbose, log=log)
+    if verbose: log.write(" - Merging sumstats using chr and pos...")
+    merged_sumstats = _quick_merge_sumstats(sumstats1=sumstats1,sumstats2=sumstats2)
+    
+    del(sumstats1)
+    del(sumstats2)
+    garbage_collect.collect()
+    
+    ######################################################################################################################
+    #process highlight and pinpoint id
     for i in range(len(highlight)):
         highlight1.append(highlight[i])
         highlight2.append(highlight[i])
@@ -125,128 +188,27 @@ def plot_miami(
     for i in range(len(anno_set2)):
         anno_set2[i] = anno_set2[i][0] * large_number + anno_set2[i][1]
 
-        
-        
-        
-    if verbose: log.write(" -Loading sumstats1:" + path1)
-    if verbose: log.write(" -Sumstats1 CHR,POS,P information will be obtained from:",cols1)
-    sumstats1 = pd.read_csv(path1,sep=sep[0],usecols=cols1,dtype={cols1[0]:"string",cols1[1]:"Int64",cols1[2]:"float64"},**readcsv_args)
-    sumstats1 = sumstats1.rename(columns={cols1[0]:"CHR",cols1[1]:"POS",cols1[2]:"P"})
-    sumstats1["CHR"] = np.floor(pd.to_numeric(sumstats1["CHR"].map(get_chr_to_number(),na_action="ignore"), errors='coerce')).astype('Int64')
-    sumstats1["POS"] = np.floor(pd.to_numeric(sumstats1["POS"], errors='coerce')).astype('Int64')   
-    
-    ## load sumstats2 ###########################################################################################################
-    if verbose: log.write(" -Loading sumstats2:" + path2)
-    if verbose: log.write(" -Sumstats2 CHR,POS,P information will be obtained from:",cols2)
-    sumstats2 = pd.read_csv(path2,sep=sep[1],usecols=cols2,dtype={cols1[0]:"string",cols1[1]:"Int64",cols1[2]:"float64"},**readcsv_args)
-    sumstats2 = sumstats2.rename(columns={cols2[0]:"CHR",cols2[1]:"POS",cols2[2]:"P"})
-    sumstats2["CHR"] = np.floor(pd.to_numeric(sumstats2["CHR"].map(get_chr_to_number(),na_action="ignore"), errors='coerce')).astype('Int64')
-    sumstats2["POS"] = np.floor(pd.to_numeric(sumstats2["POS"], errors='coerce')).astype('Int64')
-    
-    ## create merge index
-    sumstats1["TCHR+POS"]=sumstats1["CHR"]*large_number + sumstats1["POS"]
-    sumstats2["TCHR+POS"]=sumstats2["CHR"]*large_number + sumstats2["POS"]
-    sumstats1["TCHR+POS"] = sumstats1["TCHR+POS"].astype('Int64')
-    sumstats2["TCHR+POS"] = sumstats2["TCHR+POS"].astype('Int64')
-    sumstats1.drop(labels=["CHR","POS"],axis=1,inplace=True)
-    sumstats2.drop(labels=["CHR","POS"],axis=1,inplace=True)
-    sumstats1.dropna(inplace=True)
-    sumstats2.dropna(inplace=True)
     ## merging ###########################################################################################################
-    if verbose: log.write(" - Merging sumstats using chr and pos...")
-    merged_sumstats = pd.merge(sumstats1,sumstats2,on="TCHR+POS",how="outer",suffixes=('_1', '_2'))
-    merged_sumstats["CHR"] = np.divmod(merged_sumstats["TCHR+POS"],large_number)[0]
-    merged_sumstats["POS"] = np.divmod(merged_sumstats["TCHR+POS"],large_number*merged_sumstats["CHR"])[1]
-    
-    del(sumstats1)
-    del(sumstats2)
-    garbage_collect.collect()
-    
-    chrom = "CHR"
-    pos="POS"
-    
-    merged_sumstats["scaled_P_1"] = -np.log10(merged_sumstats["P_1"])
-    merged_sumstats["scaled_P_2"] = -np.log10(merged_sumstats["P_2"])
     
     if skip >0:
         sumstats = merged_sumstats.loc[(merged_sumstats["scaled_P_1"]>skip) | ( merged_sumstats["scaled_P_2"]>skip),:]  
     else:
         sumstats = merged_sumstats
     
-    
-    
-    
-    
     if region is not None:
-        region_chr = region[0]
-        region_start = region[1]
-        region_end = region[2]
         marker_size=(25,45)
-        if verbose:log.write(" -Extract SNPs in region : chr"+str(region_chr)+":"+str(region[1])+"-"+str(region[2])+ "...")
-        in_region_snp = (sumstats[chrom]==region_chr) &(sumstats[pos]<region_end) &(sumstats[pos]>region_start)
-        if verbose:log.write(" -Extract SNPs in specified regions: "+str(sum(in_region_snp)))
-        sumstats = sumstats.loc[in_region_snp,:]
+        sumstats = _quick_extract_snp_in_region(sumstats,region, verbose=verbose, log=log)
 
+    # assign i and get tick dictionary
+    sumstats, chrom_df = _quick_assign_i(sumstats)
     
-    
-    
-    
-    
-    
-    
-    sumstats = sumstats.sort_values([chrom,pos])
-    sumstats["id"]=range(len(sumstats))
-    sumstats=sumstats.set_index("id")
-
-    #create a df , groupby by chromosomes , and get the maximum position
-    posdic = sumstats.groupby(chrom)[pos].max()
-    # convert to dictionary
-    posdiccul = dict(posdic)
-
-    # fill empty chr with 0
-    for i in range(0,26):
-        if i in posdiccul: continue
-        else: posdiccul[i]=0
-
-    # cumulative sum dictionary
-    for i in range(2,sumstats[chrom].max()+1):
-        posdiccul[i]= posdiccul[i-1] + posdiccul[i] + sumstats[pos].max()*0.05
-
-    # convert base pair postion to x axis position using the cumulative sum dictionary
-    sumstats["add"]=sumstats[chrom].apply(lambda x : posdiccul[int(x)-1])
-    sumstats["i"]=sumstats[pos]+sumstats["add"]
-
-    #for plot, get the chr text tick position      
-    chrom_df=sumstats.groupby(chrom)['i'].agg(lambda x: (x.min()+x.max())/2)
-    sumstats["i"] = np.floor(pd.to_numeric(sumstats["i"], errors='coerce')).astype('Int64')
-    
-    
-    
-    ## highlight
-    if len(highlight1)>0 :
-        to_highlight1 = sumstats.loc[sumstats["TCHR+POS"].isin(highlight1),:]
-        #assign colors: 0 is hightlight color
-        if len(to_highlight1)>0:
-            for i,row in to_highlight1.iterrows():
-                target_chr = int(row[chrom])
-                target_pos = int(row[pos])
-                right_chr = sumstats["CHR"]==target_chr  
-                up_pos = sumstats["POS"]>target_pos-highlight_windowkb*1000
-                low_pos = sumstats["POS"]<target_pos+highlight_windowkb*1000
-                sumstats.loc[right_chr&up_pos&low_pos,"HUE1"]="0"
-    
-    if len(highlight2)>0:
-        to_highlight2 = sumstats.loc[sumstats["TCHR+POS"].isin(highlight2),:]
-        if len(to_highlight2)>0:
-            for i,row in to_highlight2.iterrows():
-                target_chr = int(row[chrom])
-                target_pos = int(row[pos])
-                right_chr = sumstats["CHR"]==target_chr  
-                up_pos = sumstats["POS"]>target_pos-highlight_windowkb*1000
-                low_pos = sumstats["POS"]<target_pos+highlight_windowkb*1000
-                sumstats.loc[right_chr&up_pos&low_pos,"HUE2"]="0"
-    
-    
+    ## assign highlight indicator
+    if len(highlight1)>0 or len(highlight2)>0:
+        sumstats, to_highlight1, to_highlight2 = _quick_assign_highlight_hue_pair(sumstats = sumstats, 
+                                                    highlight1 = highlight1, 
+                                                    highlight2 = highlight2, 
+                                                    highlight_windowkb = highlight_windowkb,
+                                                    verbose=verbose, log=log)
     
     if verbose: log.write("Plotting...")
     ## figure ###########################################################################################################
@@ -257,24 +219,16 @@ def plot_miami(
     plt.subplots_adjust(hspace=region_hspace)
     
     ###########################################################################################################
-    
-    sumstats["s1"]=1
-    sumstats["s2"]=1
-    sumstats.loc[sumstats["scaled_P_1"]>-np.log10(5e-4),"s1"]=2
-    sumstats.loc[sumstats["scaled_P_1"]>-np.log10(suggestive_sig_level),"s1"]=3
-    sumstats.loc[sumstats["scaled_P_1"]>-np.log10(sig_level),"s1"]=4
-    sumstats.loc[sumstats["scaled_P_2"]>-np.log10(5e-4),"s2"]=2
-    sumstats.loc[sumstats["scaled_P_2"]>-np.log10(suggestive_sig_level),"s2"]=3
-    sumstats.loc[sumstats["scaled_P_2"]>-np.log10(sig_level),"s2"]=4
-    
+    sumstats["s1"] = _quick_assign_marker_relative_size(sumstats["scaled_P_1"])
+    sumstats["s2"] = _quick_assign_marker_relative_size(sumstats["scaled_P_2"])
     
     sumstats["chr_hue"]=sumstats[chrom].astype("category")
     
 
     ##########################################################################################################################
-    maxy = max(sumstats["scaled_P_1"].max(), sumstats["scaled_P_2"].max())
-    maxy1 = sumstats["scaled_P_1"].max()
-    maxy5 = sumstats["scaled_P_2"].max()
+    maxy = max(sumstats["scaled_P_1"].max(skipna=True), sumstats["scaled_P_2"].max(skipna=True))
+    maxy1 = sumstats["scaled_P_1"].max(skipna=True)
+    maxy5 = sumstats["scaled_P_2"].max(skipna=True)
     
     if cut:
         if cut is True:
@@ -287,22 +241,20 @@ def plot_miami(
                 cutfactor = ( maxy - cut )/5
         if cut:
             if verbose: log.write(" - Minus log10(P) values above " + str(cut)+" will be shrunk with a shrinkage factor of " + str(cutfactor)+"...")
-
             maxticker=int(max(np.round(sumstats["scaled_P_1"].max(skipna=True)) , np.round(sumstats["scaled_P_2"].max(skipna=True))))
-            
             sumstats.loc[sumstats["scaled_P_1"]>cut,"scaled_P_1"] = (sumstats.loc[sumstats["scaled_P_1"]>cut,"scaled_P_1"]-cut)/cutfactor +  cut
             sumstats.loc[sumstats["scaled_P_2"]>cut,"scaled_P_2"] = (sumstats.loc[sumstats["scaled_P_2"]>cut,"scaled_P_2"]-cut)/cutfactor +  cut
             maxy = (maxticker-cut)/cutfactor + cut
-            maxy1=( int(np.round(sumstats["scaled_P_1"].max(skipna=True))) -cut)/cutfactor + cut
-            maxy5=( int(np.round(sumstats["scaled_P_2"].max(skipna=True))) -cut)/cutfactor + cut
+            maxy1=( (sumstats["scaled_P_1"].max(skipna=True)) -cut)/cutfactor + cut
+            maxy5=( (sumstats["scaled_P_2"].max(skipna=True)) -cut)/cutfactor + cut
     ##########################################################################################################################
-    legend = None
+    legend=None
     style=None
     linewidth=0
     
     
     palette = sns.color_palette(colors1,n_colors=sumstats[chrom].nunique())  
-    plot = sns.scatterplot(data=sumstats, x='i', y='scaled_P_1',
+    plot1 = sns.scatterplot(data=sumstats, x='i', y='scaled_P_1',
         hue='chr_hue',
         palette= palette,
         legend=legend,
@@ -312,7 +264,7 @@ def plot_miami(
         linewidth=linewidth,
         zorder=2,ax=ax1,edgecolor="black")     
     palette = sns.color_palette(colors2,n_colors=sumstats[chrom].nunique())  
-    plot = sns.scatterplot(data=sumstats, x='i', y='scaled_P_2',
+    plot2 = sns.scatterplot(data=sumstats, x='i', y='scaled_P_2',
         hue='chr_hue',
         palette= palette,
         legend=legend,
@@ -322,6 +274,7 @@ def plot_miami(
         linewidth=linewidth,
         zorder=2,ax=ax5,edgecolor="black") 
     
+    highlight_i = pd.DataFrame()
     if len(highlight1)>0 :
         if len(to_highlight1)>0:
             if verbose: log.write(" -Highlighting target loci for sumstats1...")
@@ -389,7 +342,7 @@ def plot_miami(
     for ax in [ax1,ax5]:
         if cut == 0: 
             ax.set_ylim(skip,maxy*1.2)
-        if cut:
+        if cut!= 0:
             cutline = ax.axhline(y=cut, linewidth = 2,linestyle="--",color=cut_line_color,zorder=1)
             if ((maxticker-cut)/cutfactor + cut) > cut:
                 ax.set_yticks([x for x in range(skip,cut+1,2)]+[(maxticker-cut)/cutfactor + cut])
@@ -406,7 +359,7 @@ def plot_miami(
         if len(anno_set1)>0:
             to_annotate1=sumstats.loc[sumstats["TCHR+POS"].isin(anno_set1),:]
         if len(anno_set2)>0:
-            to_annotate2=sumstats.loc[sumstats["TCHR+POS"].isin(anno_set2),:]
+            to_annotate5=sumstats.loc[sumstats["TCHR+POS"].isin(anno_set2),:]
     elif anno is not None:
         to_annotate1 = getsig(sumstats.loc[sumstats["scaled_P_1"]> float(-np.log10(sig_level)),:],
                        "TCHR+POS",
@@ -419,7 +372,6 @@ def plot_miami(
                        verbose=False,
                        sig_level=sig_level)
 
-
         to_annotate5 = getsig(sumstats.loc[sumstats["scaled_P_2"]> float(-np.log10(sig_level)),:],
                        "TCHR+POS",
                        "CHR",
@@ -454,153 +406,39 @@ def plot_miami(
 ####################################################################################################################
 
 # Add Annotation to manhattan plot #######################################################
-           ## final
-    if anno is not None:
-        for index,ax,to_annotate,anno_d, anno_alias in [(0,ax1,to_annotate1,anno_d1,anno_alias1),(1,ax5,to_annotate5,anno_d2,anno_alias2)]:
-            ###################### annotate() args
-            fontweight = "normal"
-
-            
-            anno_default = {"rotation":40,"style":"italic","ha":"left","va":"bottom","fontsize":fontsize,"fontweight":fontweight}
-            ########################
-            
-            if index==0:
-                to_annotate["scaled_P"] = to_annotate1["scaled_P_1"]
-                maxy = maxy1
-            else:
-                to_annotate["scaled_P"] = to_annotate5["scaled_P_2"]
-                anno_default["rotation"] = -40
-                anno_default["ha"]="left"
-                anno_default["va"]="top"
-                maxy = maxy5
-            
-            snpid = "TCHR+POS"
-
-            if (anno is not None) and (to_annotate.empty!=True):
-                #initiate a list for text and a starting position
-                text = []
-                last_pos=0
-                anno_count=0
-
-                ## log : annotation column
-                if anno==True:
-                        annotation_col="CHR:POS"
-                elif anno is not None:
-                        annotation_col = anno
-                if verbose: log.write(" -Annotating using column "+annotation_col+"...")
-
-                ##   
-                for rowi,row in to_annotate.iterrows():
-                    # avoid text overlapping
-                    ## calculate y span
-                    
-                    if region is not None:
-                        y_span = region[2] - region[1]
-                    else:
-                        y_span = sumstats["i"].max()-sumstats["i"].min()
-
-                    ## adjust x to avoid overlapping
-                    if row["i"]>last_pos+repel_force*y_span:
-                        last_pos=row["i"]
-                    else:
-                        last_pos+=repel_force*y_span
-
-                    if arm_scale_d is not None:
-                        if anno_count not in arm_scale_d.keys():
-                            arm_scale =1
-                        else:
-                            arm_scale = arm_scale_d[anno_count]
-
-                    # vertical arm length in pixels
-                    armB_length_in_point = plot.transData.transform((skip,1.15*maxy))[1]-plot.transData.transform((skip, row["scaled_P"]+1))[1]-arm_offset/2
-                    #
-                    armB_length_in_point = armB_length_in_point*arm_scale
-                    
-                    if arm_scale>=1:
-                        armB_length_in_point= armB_length_in_point if armB_length_in_point>0 else plot.transData.transform((skip, maxy+2))[1]-plot.transData.transform((skip,  row["scaled_P"]+1))[1] 
-
-                    if anno_fixed_arm_length is not None:
-                        anno_fixed_arm_length_factor = plot.transData.transform((skip,anno_fixed_arm_length))[1]-plot.transData.transform((skip,0))[1] 
-                        armB_length_in_point = anno_fixed_arm_length_factor
-                    
-                    if anno==True:
-                        if row[snpid] in anno_alias.keys():
-                            annotation_text = anno_alias[row[snpid]]
-                        else:
-                            annotation_text="Chr"+ str(row[chrom]) +":"+ str(int(row[pos]))
-                    elif anno == "GENENAME":
-                        annotation_text=row["GENENAME"]
-
-                    # annoatte position
-                    xy=(row["i"],row["scaled_P"]+0.2)  
-                    
-                    xytext=(last_pos,1.15*maxy*arm_scale)
-                    
-                    if anno_fixed_arm_length is not None:
-                        armB_length_in_point = anno_fixed_arm_length
-                        xytext=(row["i"],row["scaled_P"]+0.2+anno_fixed_arm_length)
-                    
-                    if anno_count not in anno_d.keys():
-                        if index==0:
-                            arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
-                                                connectionstyle="arc,angleA=0,armA=0,angleB=90,armB="+str(armB_length_in_point)+",rad=0")
-                        else:
-                            arrowargs = dict(arrowstyle="-|>",relpos=(0,1),color="#ebebeb",
-                                                connectionstyle="arc,angleA=0,armA=0,angleB=-90,armB="+str(armB_length_in_point)+",rad=0")
-                    
-                    else:
-                        xy=(row["i"],row["scaled_P"])
-                        if anno_d[anno_count] in ["right","left","l","r"]:
-                            if anno_d[anno_count]=="right" or anno_d[anno_count]=="r": 
-                                armoffsetall = (plot.transData.transform(xytext)[0]-plot.transData.transform(xy)[0])*np.sqrt(2)
-                                armoffsetb = arm_offset 
-                                armoffseta = armoffsetall - armoffsetb   
-                                arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
-                                                    connectionstyle="arc,angleA=-135,armA="+str(armoffseta)+",angleB=45,armB="+str(armoffsetb)+",rad=0")
-                            elif anno_d[anno_count]=="left" or anno_d[anno_count]=="l":
-                                arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
-                                                    connectionstyle="arc,angleA=-135,armA="+str(arm_offset)+",angleB=135,armB="+str(arm_offset)+",rad=0")
-                        else:
-                            if anno_d[anno_count][0]=="right" or anno_d[anno_count][0]=="r": 
-                                armoffsetall = (plot.transData.transform(xytext)[0]-plot.transData.transform(xy)[0])*np.sqrt(2)
-                                armoffsetb = anno_d[anno_count][1] 
-                                armoffseta = armoffsetall - armoffsetb   
-                                arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
-                                                    connectionstyle="arc,angleA=-135,armA="+str(armoffseta)+",angleB=45,armB="+str(armoffsetb)+",rad=0")
-                            elif anno_d[anno_count]=="left" or anno_d[anno_count]=="l":
-                                arrowargs = dict(arrowstyle="-|>",relpos=(0,0),color="#ebebeb",
-                                                    connectionstyle="arc,angleA=-135,armA="+str( anno_d[anno_count][1])+",angleB=135,armB="+str( anno_d[anno_count][1])+",rad=0")
-
-                    bbox_para=None            
-                    
-                    if len(highlight) >0:
-                        if row["i"] in highlight_i:
-                            anno_default["fontweight"] = "bold"
-                        else:
-                            anno_default["fontweight"] = "normal"
-                    
-                    for key,value in anno_args.items():
-                        anno_default[key]=value
-
-                    ax.annotate(annotation_text,
-                            xy=xy,
-                            xytext=xytext,
-                            bbox=bbox_para,
-                            arrowprops=arrowargs,
-                            zorder=100,
-                            **anno_default
-                            )
-
-                    anno_count +=1
-
-
-            else:
-                if verbose: log.write(" -Skip annotating")
-    else:
-        if verbose: log.write(" -Skip annotating")
+    ax1, ax5 = annotate_pair(
+                                sumstats=sumstats,
+                                anno=anno,
+                                ax1=ax1,
+                                ax5=ax5,
+                                highlight_i=highlight,
+                                to_annotate1=to_annotate1,
+                                to_annotate5=to_annotate5,
+                                anno_d1=anno_d1,
+                                anno_d2=anno_d2,
+                                anno_alias1=anno_alias1,
+                                anno_alias2=anno_alias2,
+                                anno_style=anno_style,
+                                anno_args=anno_args,
+                                anno_max_iter=anno_max_iter,
+                                anno_adjust=anno_adjust,
+                                arm_scale=arm_scale,
+                                arm_scale_d=arm_scale_d,
+                                anno_fixed_arm_length=anno_fixed_arm_length,
+                                arm_offset=arm_offset,
+                                maxy1=maxy1,
+                                maxy5=maxy5,
+                                fontsize=fontsize,
+                                region=region,
+                                skip=skip,
+                                chrom="CHR",
+                                pos="POS",
+                                repel_force=repel_force,
+                                verbose=verbose,
+                                log=log)
 
 ####################################################################################################################    
-           
+# Adjust the visibility for spines #######################################################
     ax1.spines["top"].set_visible(False)
     ax1.spines["right"].set_visible(False)
     ax1.spines["left"].set_visible(True)
@@ -612,7 +450,6 @@ def plot_miami(
     ax5.spines["bottom"].set_visible(False)
 ####################################################################################################################
 
-
     if region is not None:
         most_left_snp = sumstats["i"].idxmin()            
         gene_track_offset = sumstats.loc[most_left_snp,pos]-region[1]
@@ -636,7 +473,8 @@ def plot_miami(
             # set x ticks m plot
         ax1.set_xticks(np.linspace(gene_track_start_i+region[1], gene_track_start_i+region[2], num=region_step))
         ax5.set_xticks(np.linspace(gene_track_start_i+region[1], gene_track_start_i+region[2], num=region_step))
-        ax1.set_xticklabels(region_ticks,rotation=45,fontsize=fontsize,family="sans-serif")
+        ax1.set_xticklabels(region_ticks,rotation=30,fontsize=fontsize,family="sans-serif")
+        fig.tight_layout()
 
         ax1.set_xlim([gene_track_start_i+region[1], gene_track_start_i+region[2]])
         ax5.set_xlim([gene_track_start_i+region[1], gene_track_start_i+region[2]])
